﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using SilverClickableArea2;
using System.Windows.Browser;

namespace SilverTicTacToe2
{
    [ScriptableType]
    public partial class Page : UserControl
    {
        public Page()
        {
            InitializeComponent();
            HtmlPage.RegisterScriptableObject("SilverTicTacToe2", this);
        }


        #region Fields

        private string _win = "You win!";
        private string _lose = "You lose!";
        private string _equal = "No winner this time!";
        private string _wait = "Wait for opponents move";
        private string _move = "Your move";
        private string _start = "Play against the Computer";
        private string _end = "Click this to play again";

        private int _moveCount = 0;
        private bool _crossFirst = false;
        private int[] _board;
        private bool _isCross = true;
        private bool _gameEnd = false;

        #endregion

        #region Events

        public void Page_Loaded(object o, EventArgs e)
        {
            // Required to initialize variables
            InitializeComponent();
            StartGame(true);
        }

        public void area_Clicked(SilverArea2 sa)
        {
            if (_isCross) ExecuteClick(sa);
        }

        void txtTopText_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            txtStatus1.Text = _start;
            txtStatus1.MouseLeftButtonDown -= new MouseButtonEventHandler(txtTopText_MouseLeftButtonDown);
            txtStatus1.Cursor = Cursors.Arrow;
            StartGame();
        }

        #endregion

        #region Functions

        private bool CheckWin(int who)
        {
            if ((_board[0] == who) && (_board[1] == who) && (_board[2] == who)) return true;
            if ((_board[0] == who) && (_board[3] == who) && (_board[6] == who)) return true;
            if ((_board[0] == who) && (_board[4] == who) && (_board[8] == who)) return true;
            if ((_board[1] == who) && (_board[4] == who) && (_board[7] == who)) return true;
            if ((_board[2] == who) && (_board[4] == who) && (_board[6] == who)) return true;
            if ((_board[2] == who) && (_board[5] == who) && (_board[8] == who)) return true;
            if ((_board[3] == who) && (_board[4] == who) && (_board[5] == who)) return true;
            if ((_board[6] == who) && (_board[7] == who) && (_board[8] == who)) return true;
            return false;
        }

        private void GameEnd()
        {
            _gameEnd = true;
            txtStatus1.Text = _end;
            txtStatus1.MouseLeftButtonDown += new MouseButtonEventHandler(txtTopText_MouseLeftButtonDown);
            txtStatus1.Cursor = Cursors.Hand;
        }

        private void StartGame()
        {
            StartGame(false);
        }

        private void StartGame(bool initial)
        {
            _crossFirst = !_crossFirst;
            if (_crossFirst) _isCross = true;
            else _isCross = false;
            _moveCount = 0;
            _board = new int[9];
            txtStatus1.Text = _start;
            for (int i = 0; i < _board.Length; i++)
            {
                _board[i] = 0;
                int position = i + 1;
                SilverArea2 sa = this.FindName("area" + position.ToString()) as SilverArea2;
                if (initial) sa.AreaClicked += new AreaClick(area_Clicked);
                sa.Position = position;
                sa.Reset();
            }
            _gameEnd = false;
            SetStatus();
            if (!_isCross) ComputerPlayer();
        }

        private void SetStatus()
        {
            if (!_gameEnd)
            {
                if (_isCross) txtStatus2.Text = _move;
                else txtStatus2.Text = _wait;
            }
        }

        private void ExecuteClick(SilverArea2 sa)
        {
            if (!_gameEnd && _board[sa.Position - 1] == 0)
            {
                int who = _isCross ? 1 : 2;
                _board[sa.Position - 1] = who;
                if (_isCross) sa.ExposeSymbol(Symbol.X);
                else sa.ExposeSymbol(Symbol.O);
                _moveCount++;
                if (_moveCount > 4 && CheckWin(who))
                {
                    if (who == 1) txtStatus2.Text = _win;
                    else txtStatus2.Text = _lose;
                    GameEnd();
                }
                else if (_moveCount == 9)
                {
                    txtStatus2.Text = _equal;
                    GameEnd();
                }
                _isCross = !_isCross;
                SetStatus();
                if (!_isCross) ComputerPlayer();
            }
        }

        [ScriptableMember]
        public event EventHandler GoBrowser;

        private void ComputerPlayer()
        {
            GoBrowser(this, new EventArgs()); /* My dirty solution... */
        }

        [ScriptableMember]
        public void ResumePlaying()
        {
            SilverArea2 sa = this.FindName("area" + CalculateMove().ToString()) as SilverArea2;
            ExecuteClick(sa);
        }

        private int CalculateMove()
        {
            if (_board[4] == 0) return 5;
            if (_board[0] == 0) return 1;
            if (_moveCount >= 3)
            {
                //Possible to win
                int win = FindWin(2);
                if (win != -1) return win + 1;
                //Possible to lose
                win = FindWin(1);
                if (win != -1) return win + 1;
            }
            for (int i = 0; i < _board.Length; i += 2) //first try the corners
            {
                if (_board[i] == 0)
                {
                    return i + 1;
                }
            }
            for (int i = 0; i < _board.Length; i++)
            {
                if (_board[i] == 0)
                {
                    return i + 1;
                }
            }
            return 9; /* Just to satisfy the compiler */
        }

        private int FindWin(int who)
        {
            int result = CheckPossibleWin(0, who, who);
            if (result == -1) result = CheckPossibleWin(who, 0, who);
            if (result == -1) result = CheckPossibleWin(who, who, 0);
            return result;
        }

        private int CheckPossibleWin(int x, int y, int z)
        {
            if ((_board[0] == x) && (_board[1] == y) && (_board[2] == z)) return x == 0 ? 0 : y == 0 ? 1 : 2;
            if ((_board[0] == x) && (_board[3] == y) && (_board[6] == z)) return x == 0 ? 0 : y == 0 ? 3 : 6;
            if ((_board[0] == x) && (_board[4] == y) && (_board[8] == z)) return x == 0 ? 0 : y == 0 ? 4 : 8;
            if ((_board[1] == x) && (_board[4] == y) && (_board[7] == z)) return x == 0 ? 1 : y == 0 ? 4 : 7;
            if ((_board[2] == x) && (_board[4] == y) && (_board[6] == z)) return x == 0 ? 2 : y == 0 ? 4 : 6;
            if ((_board[2] == x) && (_board[5] == y) && (_board[8] == z)) return x == 0 ? 2 : y == 0 ? 5 : 8;
            if ((_board[3] == x) && (_board[4] == y) && (_board[5] == z)) return x == 0 ? 3 : y == 0 ? 4 : 5;
            if ((_board[6] == x) && (_board[7] == y) && (_board[8] == z)) return x == 0 ? 6 : y == 0 ? 7 : 8;
            return -1;
        }

        #endregion

    }
}
